"""Entry point.

"""
from os.path import join, expanduser

from cml.usecases.query import PreprocessUsecase, KnowledgeSearchUsecase
from cml.ports.source_adapters import PandasAdapter
from cml.shared.settings import Settings
from cml.shared.settings import specific_settings_factory, read_settings
from cml.shared.request import PreprocessRequest


__all__ = (
    "load_settings",
    "get_settings",
    "get_data_source",
    "construction",
    "reconstruction",
    "reconstruction",
    "search_knowledge",
    "visualizer",
    "save_knowledge",
    "load_knowledge"
)


def default_path(func):
    def wrapper(path: str = None):
        if not path:
            path = join(expanduser("~"), ".cml", "settings.ini")

        try:
            func(path)
        except FileNotFoundError as e:
            # TODO (dmt):Provide proper exception handling!
            pass
    return wrapper


@default_path
def load_settings(path: str):
    read_settings(path)


def get_settings():
    return Settings


def get_data_source():
    general_settings = specific_settings_factory("general")
    preprocessing_settings = specific_settings_factory("preprocessing")
    block_processing_settings = specific_settings_factory("block_processing")
    source_adapter = PandasAdapter.read_csv_data(general_settings.input_file)
    preprocessing_req = PreprocessRequest(source_adapter,
                                          preprocessing_settings,
                                          block_processing_settings)
    preprocessing_usecase = PreprocessUsecase()
    return preprocessing_usecase.execute(preprocessing_req)


def construction():
    pass


def reconstruction():
    pass


def search_knowledge():
    pass


def visualizer():
    pass


def save_knowledge():
    pass


def load_knowledge():
    pass
